#!/usr/bin/env python
"""Bashplot: Instant data plotting from the terminal into the terminal."""


######################################################
#
# bashplot - instant plotting via the command line
# written by Anselm Hahn (Anselm.Hahn@gmail.com)
# inspired by Benjamin Gleitzman (gleitz@mit.edu)
#
######################################################

import argparse
import sys
from pathlib import Path

import numpy as np
import plotille as plt

from . import __version__


def log(msg, mode=None):
    """Print messages to display.

    Parameters
    ----------
    msg : str
        Message to print to the terminal.
    mode : int, optional
        If mode is activated, message becomes an error message.
    """
    if mode:
        print(f"[ERROR] {msg}")
    else:
        print(msg)


def load_data(fname, args):
    """Load data via np.genfromtxt.

    Parameters
    ----------
    fname : str
        Filename to load via np.genfromtxt.
    args : dict
        Dictionary of the keywords and values from the parser.

    Returns
    -------
    data : float-array
        Returns a 2D-Numpy-array with `dtype=float`.
    """
    return np.genfromtxt(
        fname,
        dtype=np.float,
        comments=args["comments"],
        delimiter=args["delimiter"],
        skip_header=args["skip_header"],
        skip_footer=args["skip_footer"],
        usecols=args["usecols"],
    )


def plot_plot(fig, x, Y, label):
    """Make the plot-figures.

    plot_plot() is generating a single- or multi-plot by recursiving calling plot().

    Parameters
    ----------
    fig : class
        Figure class for the terminal plot
    x : float-array
        1D-Numpy-array with the float column x-values.
    Y : float-array
        1D- or 2D-Numpy-array with the float column Y-values.
    label : str
        The label of the plot(s) is the current filename.

    Returns
    -------
    fig :  class
        Updated figure class for the terminal plot
    plot_plot() : function
        Returns the function itself for a smaller (n-1) float-array (Y) until it is an
        1D-array.
    """
    fig.plot(x, Y[:, 0], label=label)
    if Y.shape[1] == 1:
        return fig
    return plot_plot(fig, x, Y[:, 1:], label=label)


def plot_scatter(fig, x, Y, label):
    """Make the scatter-figures.

    plot_scatter() is generating a single- or multi-plot by recursive calling
    scatter().

    Parameters
    ----------
    fig : class
        Figure class for the terminal plot
    x : float-array
        1D-Numpy-array with the float column x-values.
    Y : float-array
        1D- or 2D-Numpy-array with the float column Y-values.
    label : str
        The label of the scatter-plot(s) is the current filename.

    Returns
    -------
    fig :  class
        Updated figure class for the terminal plot
    plot_scatter() : function
        Returns the function itself for a smaller (n-1) float-array (Y) until it is an
        1D-array.
    """
    fig.scatter(x, Y[:, 0], label=label)
    if Y.shape[1] == 1:
        return fig
    return plot_plot(fig, x, Y[:, 1:], label=label)


def plot(data, args, label):
    """Generate the plots as classical or scatter plots.

    plot() is generating the classical or scatter plots according to the arguments
    `args`.

    Parameters
    ----------
    data : float-array
        2D-Numpy-array with `dtype=float`.
    args : dict
        Dictionary of the keywords and values from the parser.
    label : str
        The label of the scatter-plot(s) is the current filename.
    """
    fig = plt.Figure()
    fig.widht = args["size"][0]
    fig.height = args["size"][1]
    try:

        if args["x_limits"]:
            fig.set_x_limits(min_=args["x_limits"][0], max_=args["x_limits"][1])
        else:
            fig.set_x_limits(min_=np.min(data[:, 0]), max_=np.max(data[:, 0]))

        if args["y_limits"]:
            fig.set_y_limits(min_=args["y_limits"][0], max_=args["y_limits"][1])
        else:
            try:
                fig.set_y_limits(min_=np.min(data[:, 1:]), max_=np.max(data[:, 1:]))
            except ValueError:
                min_max = np.mean(data[:, 1:])
                fig.set_y_limits(
                    min_=min_max - min_max * 0.1, max_=min_max + min_max * 0.1
                )
        if args["color"]:
            fig.color_mode = "rgb"

        x = data[:, 0]
        y = data[:, 1:]

        if args["scatter"]:
            fig = plot_scatter(fig, x=x, Y=y, label=label)
        else:
            fig = plot_plot(fig, x=x, Y=y, label=label)

        if args["legend"]:
            log(fig.show(legend=True))
        else:
            log(fig.show(legend=False))
    except IndexError:
        log(f"corrupted data in {label}", mode=1)
        sys.exit(1)


def bashplot(fnames, args):
    """bashplot.

    bashplot() is plotting each file independently according to the args. For a
    filename list >1, bashplot() is calling itself again by reducing the list by
    the value of -1.

    Parameters
    ----------
    fnames : str-list
        List of the filename(s); is always a list even if single value included.
    args : dict
        Dictionary of the keywords and values from the parser.

    Returns
    -------
    bashplot() : function
        Returns the function itself for smaller (n-1) list of filenames (fnames).
    """
    if len(fnames) == 1:
        data = load_data(fname=Path(fnames[0]), args=args)
        plot(data, args, label=fnames[0])
    else:
        data = load_data(fname=Path(fnames[0]), args=args)
        plot(data, args, label=fnames[0])
        return bashplot(fnames[1:], args)


def get_args(opt=None):
    """Get the parser arguments from the command line.

    Parameters
    ----------
    opt : dict, optional
        Optional Dictionary for modifying the parser arguments; default is None.

    Returns
    -------
    args : dict
        Dictionary of the keywords and values from the parser.
    """
    parser = argparse.ArgumentParser(
        description=("Instant data plotting from the terminal into the terminal")
    )
    # Arguments for loading the data
    parser.add_argument(
        "infile", nargs="*", type=str, help="load data file(s) as ASCII"
    )
    parser.add_argument(
        "-cm",
        "--comments",
        help=(
            "define the character to indicate the start of the document; "
            "default is None"
        ),
        default=None,
        type=str,
    )
    parser.add_argument(
        "-d",
        "--delimiter",
        help=("define the type of deilimiter; default is any consecutive 'whitespace'"),
        default=None,
        type=str,
    )
    parser.add_argument(
        "-skh",
        "--skip_header",
        help=(
            "define the number of skipped rows from the beginning of the document; "
            "default is 0"
        ),
        default=0,
        type=int,
    )
    parser.add_argument(
        "-skf",
        "--skip_footer",
        help=(
            "define the number of skipped rows from the end of the document; "
            "default is 0"
        ),
        default=0,
        type=int,
    )
    parser.add_argument(
        "-col",
        "--usecols",
        help=(
            "define the pair of x- and y-columns for the data plot; default is None. "
            "This  means all available will be loaded; otherwise tip a series of "
            "columns started with 0 like 0, 1, 4"
        ),
        default=None,
        nargs=2,
        type=int,
    )
    # Arguments for plotting the data
    parser.add_argument(
        "-sz",
        "--size",
        help=(
            "define the width and heights of the plots; default is width = 60 and the "
            "heights = 40. The values have to be int-values."
        ),
        default=[60, 40],
        nargs=2,
        type=int,
    )
    parser.add_argument(
        "-x",
        "--x_limits",
        help=(
            "define the mimimum and maximum x-range of the plot; default is the "
            "automatic generated minimum and the maximum of the data, but can be "
            "replaced by any float-values."
        ),
        default=None,
        nargs=2,
        type=float,
    )
    parser.add_argument(
        "-y",
        "--y_limits",
        help=(
            "define the mimimum and maximum y-range of the plot; default is the "
            "automatic generated minimum and the maximum of the data, but can be "
            "replaced by any float-values."
        ),
        default=None,
        nargs=2,
        type=float,
    )
    parser.add_argument(
        "-sc",
        "--scatter",
        help=("replaced regular plot by scatter plot"),
        action="store_true",
    )
    parser.add_argument(
        "-c", "--color", help=("enable RGB colorized the plots"), action="store_true"
    )
    parser.add_argument(
        "-l", "--legend", help=("disable the legend of the plots"), action="store_false"
    )
    parser.add_argument(
        "-v",
        "--version",
        help=("displays the current version of bathchplot"),
        action="store_true",
    )
    args = vars(parser.parse_args())
    if opt:
        for item, value in opt.items():
            args[item] = value
    return args


def command_line_runner():
    """Run bashplot() via command line."""
    args = get_args()

    if args["version"]:
        log(__version__)

    if not args["infile"]:
        log("Missing input file(s)!", mode=1)
        return

    fnames = args["infile"]
    bashplot(fnames, args)


if __name__ == "__main__":
    command_line_runner()



from pathlib import Path

import numpy as np


def generate(fname="test_data"):
    """Generation of example test-sets.

    generate() generates an example test-sets for:
    1.  sin from -2 * pi to +2 * pi for sinus.
    2.  cos from -2 * pi to +2 * pi for cosinus.
    3.  random gaussian distribution

    Parameters
    ----------
    fname : str, optional
        Filename of the to exported ASCII-data. Default is `test_data.txt`.
    """

    x = np.linspace(-2 * np.pi, 2 * np.pi)

    y_1 = np.sin(x)
    data = np.array([x, y_1]).T
    np.savetxt(Path(f"./{fname}_sin.txt"), data)

    y_2 = np.cos(x)
    data = np.array([x, y_2]).T
    np.savetxt(Path(f"./{fname}_cos.txt"), data)

    data = np.array([x, y_1, y_2]).T
    np.savetxt(Path(f"./{fname}_mixed.txt"), data)

    data = np.array(
        [
            x,
            y_1,
            y_2,
            2.0 * (y_1 - y_2),
            (y_1 - y_2) ** 2,
        ]
    ).T
    np.savetxt(Path(f"./{fname}_mixed.dat"), data, delimiter="\t")

    mu, sigma = 0, 0.1  # mean and standard deviation
    x = np.linspace(-0.3, 0.3)
    y = np.random.normal(mu, sigma, 50)
    data = np.array([x, y]).T
    np.savetxt(Path(f"./{fname}_random_gaussian.txt"), data)

    np.savetxt(
        Path(f"./{fname}_equaldata.out"),
        np.array([[-1.0, 2.0], [0.0, 2.0], [1.0, 2.0]]),
    )



"""Test for basplot."""

from pathlib import Path
from unittest import mock

import pytest
import sample_generator

from bashplot import bashplot

sample_generator.generate()

print()

test_txt = list(Path.cwd().glob("test*.txt"))
test_dat = list(Path.cwd().glob("test*.dat"))
test_out = list(Path.cwd().glob("test*.out"))

args_1 = {
    "infile": test_txt,
    "comments": None,
    "delimiter": None,
    "skip_header": 0,
    "skip_footer": 0,
    "usecols": (0, 1),
    "size": [30, 20],
    "x_limits": None,
    "y_limits": None,
    "scatter": False,
    "color": False,
    "legend": True,
    "version": False,
}
args_2 = {
    "infile": test_txt,
    "comments": None,
    "delimiter": None,
    "skip_header": 2,
    "skip_footer": 2,
    "usecols": (0, 1),
    "size": [60, 40],
    "x_limits": [0.0, 3.0],
    "y_limits": [0.0, 3.0],
    "scatter": True,
    "color": True,
    "legend": False,
    "version": True,
}
# Example with different delimiter
args_3 = {
    "infile": test_dat,
    "comments": None,
    "delimiter": "\t",
    "skip_header": 2,
    "skip_footer": 2,
    "usecols": (0, 1, 2, 3),
    "size": [40, 30],
    "x_limits": [0.0, 3.0],
    "y_limits": [0.0, 3.0],
    "scatter": False,
    "color": True,
    "legend": True,
    "version": True,
}
args_4 = {
    "infile": test_out,
    "comments": None,
    "delimiter": None,
    "skip_header": 0,
    "skip_footer": 0,
    "usecols": (0, 1),
    "size": [30, 20],
    "x_limits": None,
    "y_limits": None,
    "scatter": False,
    "color": False,
    "legend": True,
    "version": False,
}


def test_fnames():
    assert bashplot.get_args(opt={"infile": test_txt})["infile"] == test_txt


def test_options_scatter():
    assert bashplot.get_args(opt={"scatter": True})["scatter"] is True


def test_options_color():
    assert bashplot.get_args(opt={"color": True})["color"] is True


def test_options_legend():
    assert bashplot.get_args(opt={"legend": False})["legend"] is False


def test_options_version():
    assert bashplot.get_args(opt={"version": True})["version"] is True


def test_xlimits():
    assert bashplot.get_args(opt={"x_limits": [0.0, 20.0]})["x_limits"] == [0.0, 20.0]


def test_ylimits():
    assert bashplot.get_args(opt={"y_limits": [0.0, 20.0]})["y_limits"] == [0.0, 20.0]


def test_size():
    assert bashplot.get_args(opt={"size": [60, 60]})["size"] == [60, 60]


def test_ranges_default():
    assert bashplot.get_args()["size"] == [60, 40]


def test_usecols():
    assert bashplot.get_args(opt={"usecols": [0, 2, 5]})["usecols"] == [0, 2, 5]


@mock.patch("bashplot.bashplot.bashplot")
def test_default_run_mock(bashplot):
    bashplot.bashplot(fnames=test_txt, args=args_1)
    assert bashplot.bashplot.is_called


def test_default_run():
    bashplot.bashplot(fnames=test_txt, args=args_1)
    assert True


@mock.patch("bashplot.bashplot.bashplot")
def test_customize_run_mock_1(bashplot):
    bashplot.bashplot(fnames=test_txt, args=args_2)
    assert bashplot.bashplot.is_called


def test_customize_run_1():
    bashplot.bashplot(fnames=test_txt, args=args_2)
    assert True


@mock.patch("bashplot.bashplot.bashplot")
def test_customize_run_mock_2(bashplot):
    bashplot.bashplot(fnames=test_dat, args=args_3)
    assert bashplot.bashplot.is_called


def test_customize_run_2():
    bashplot.bashplot(fnames=test_dat, args=args_3)
    assert True


def test_customize_run_3():
    bashplot.bashplot(fnames=test_out, args=args_4)
    assert True


@mock.patch("bashplot.bashplot.command_line_runner")
def test_command_line_mock(command_line_runner):
    bashplot.command_line_runner()
    assert bashplot.command_line_runner.is_called


def test_command_line_1():
    bashplot.command_line_runner()
    assert True


def test_log(capfd):
    bashplot.log("msg")
    out, _ = capfd.readouterr()
    assert out == "msg\n"


def test_log_error(capfd):
    bashplot.log("msg", mode=True)
    out, _ = capfd.readouterr()
    assert out == "[ERROR] msg\n"



try:
    from setuptools import find_packages, setup
except ImportError:
    from distutils.core import setup, find_packages

from pathlib import Path

import bashplot

__author__ = "Anselm Hahn"
__email__ = "Anselm.Hahn@gmail.com"


def long_description():
    #  deepcode ignore missing~close~open: <ignorable in the setup.py>
    readme = open(Path("./README.md")).read()
    #  deepcode ignore missing~close~open: <ignorable in the setup.py>
    changes = open(Path("./CHANGES.md")).read()
    #  deepcode ignore missing~close~open: <ignorable in the setup.py>
    todo = open(Path("./TODO.md")).read()

    long_description = (
        f"{readme}\n\n"
        f"CHANGES\n"
        f"-------\n"
        f"{changes}\n"
        f"TODO\n"
        f"----\n"
        f"{todo}\n"
    )
    return long_description


def requirments():
    with open("requirements.txt") as f:
        return f.read().splitlines()


setup(
    name="bashplot",
    version=bashplot.__version__,
    description="Instant data plotting from the terminal into the terminal",
    long_description=long_description(),
    long_description_content_type="text/markdown",
    install_requires=requirments(),
    packages=find_packages(),
    py_modules=[path.stem for path in Path(".").glob("bashplot/*.py")],
    author=__author__,
    author_email=__email__,
    maintainer=__author__,
    maintainer_email=__email__,
    url="https://github.com/Anselmoo/bashplot",
    license="MIT",
    entry_points={
        "console_scripts": ["bashplot = bashplot.bashplot:command_line_runner"]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Unix",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: System :: Shells",
        "Topic :: Scientific/Engineering :: Visualization",
        "Topic :: Utilities",
    ],
    keywords=[
        "terminal",
        "data-visualization",
        "data-science",
        "database",
    ],
    extras_require={"testing": ["pipenv"]},
    platforms=["Windows", "Linux", "Solaris", "Mac OS-X", "Unix"],
    test_suite="pytest",
)
